from quixote import get_session

from qommon import get_cfg
from qommon import errors
from qommon import template

import authentic.liberty.saml2 as saml2

import authentic.misc as misc

import directory

def check_access_authorizations(provider_key):
    session = get_session()
    if not session or session.user is None:
        return False

    user = get_session().get_user_object()
    if user.is_admin():
        return True

    collectivity = user.get_collectivity()
    service_instances = directory.get_service_instances(collectivity)

    providers = get_cfg('providers', {})
    accesses = user.get_as_agent().cdg59serviceAccesses or []

    for service in service_instances:
        try:
            service_type = directory.get_service(service.cdg59serviceType)
        except KeyError:
            # ignore service type removed globally
            continue

        if not service.cdg59siid in accesses:
            continue

        # handle global services
        if service_type.cdg59isGlobal:
            cdg59metadataURL = service_type.cdg59metadataURL
        else:
            cdg59metadataURL = service.cdg59metadataURL

        if not cdg59metadataURL:
            continue

        try:
            klp = [x for x, y in providers.items() if \
                  cdg59metadataURL == y.get('metadata_url')] [0]
        except IndexError:
            continue

        if provider_key == klp:
            return True

    return False


class AccessControlSpUI(saml2.SpUI):
    def _q_access(self):
        authorized = check_access_authorizations(self.provider_key)
        if not authorized:
            if get_session():
                raise errors.AccessForbiddenError()
            else:
                raise errors.AccessUnauthorizedError()

    def login(self, encryption_mode = None, method = None, nid_format = 'persistent', relay_state = None):
        return saml2.SpUI.login(self, encryption_mode, method, nid_format, 'backoffice')


class AccessControlSpDir(saml2.SpDir):
    def _q_lookup(self, component):
        return AccessControlSpUI(component)


class AlternateSaml2Directory(saml2.RootDirectory):
    sp = AccessControlSpDir()

    def check_access_authorizations(self, login):
        provider_id = login.remoteProviderId
        provider_key = misc.get_provider_key(provider_id)
        return check_access_authorizations(provider_key)

    def sso_after_authentication(self, login, user_authenticated, proxied = False):
        if user_authenticated:
            if not self.check_access_authorizations(login):
                provider_id = login.remoteProviderId
                provider_key = misc.get_provider_key(provider_id)
                try:
                    label = misc.get_provider_and_label(provider_key)[1]
                except KeyError:
                    return template.error_page(_('''\
You do not have required authorizations to access the service,
you should contact the administration of your collectivity.'''))

                return template.error_page(_('''\
You do not have required authorizations to access the "%s" service,
you should contact the administration of your collectivity.''') % label)
        return saml2.RootDirectory.sso_after_authentication(
                self, login, user_authenticated, proxied=proxied)
