import os
import time

try:
    import elementtree.ElementTree as ET
except ImportError:
    import xml.etree.ElementTree as ET

from quixote import get_request, get_session

from qommon.publisher import get_publisher, get_publisher_class

import authentic.sessions
import directory

class PraticIdPSession(authentic.sessions.BasicSession):
    def get_xml_dir(cls):
        xml_dir = os.path.join(get_publisher().app_dir, 'sessions-xml')
        if not os.path.exists(xml_dir):
            os.makedirs(xml_dir)
        return xml_dir
    get_xml_dir = classmethod(get_xml_dir)

    def store(self):
        authentic.sessions.BasicSession.store(self)
        session_xml_path = os.path.join(self.get_xml_dir(), self.id)
        if self.user:
            node = ET.Element('session')
            ET.SubElement(node, 'uid').text = self.user
            fd = file(session_xml_path, 'w')
            ET.ElementTree(node).write(fd)
            fd.close()
        else:
            if os.path.exists(session_xml_path):
                os.unlink(session_xml_path)

    def remove_object(cls, id):
        authentic.sessions.BasicSession.remove_object(id)
        session_xml_path = os.path.join(cls.get_xml_dir(), id)
        if os.path.exists(session_xml_path):
            os.unlink(session_xml_path)
    remove_object = classmethod(remove_object)


class StorageSessionManager(authentic.sessions.StorageSessionManager):
    def __init__(self, session_class = PraticIdPSession):
        authentic.sessions.StorageSessionManager.__init__(self,
                session_class = PraticIdPSession)

    def expire_session(self):
        request = get_request()
        session = get_session()
        if session.previous_user:
            user = directory.get_store().get_identity(session.previous_user)
            agent = user.get_as_agent()
            agent.cdg59lastConnectionDuration = int(time.time() - session.get_creation_time())
            try:
                agent.save(['cdg59lastConnectionDuration'])
            except:
                # too bad it failed.
                pass

        authentic.sessions.StorageSessionManager.expire_session(self)

get_publisher_class().session_manager_class = StorageSessionManager

